function []=solve_EO_ls()
%this function solves exterior orientation (EO) with least squares method

%parameters:
%i_obs=image observations
%g_obs=ground point observations
%c=camera constant
%params=a vector containing camera constant and initial values of omega, phi, kappa, X0, Y0,
%and Z0
%output parameters
%EO=a vector containing final values of omega, phi, kappa, X0, Y0, and Z0
%sigma0_2 = variance of unit weight
%res=residuals of image observations

    %load data
    g_obs=load('g_obs.dat');
    params=load('params.dat');
    i_obs=load('i_obs.dat');

    %size of image observation matrix
    size_vector = size(i_obs);

    %check the number of observations
    nof_obs = size_vector(1);

    %initial values for unknown parameters
    o = params(5);%omega
    p = params(6);%phi
    k = params(7);%kappa
    X0 = params(2);
    Y0 = params(3);
    Z0 = params(4);

    %camera constant
    c = params(1);

    %initialize
    iters = 0;
    end_t = 0;

    while end_t < 1 && iters < 300 %start iteration
    
        iters = iters + 1; %increase iters index
        
        %create design matrix A row by row
        [A,y] = create_A_matrix_and_y_vector(c, o, p, k, X0, Y0, Z0, i_obs, g_obs, nof_obs);

        %solve corrections
        x = (A'*A)\(A'*y);
        %or
        %dx = inv(A'*A)*A'*y;
        
        %display the correction vector
        disp(['Iteration round: ' num2str(iters) '. Corrections (x):']);disp(x)
   
        %update parameters with corrections
        o = o + x(1);
        p = p + x(2);
        k = k + x(3);
        X0 = X0 + x(4);
        Y0 = Y0 + x(5);
        Z0 = Z0 + x(6);
   
   %calculate residuals
        res = A*x - y;

        if max(abs(x(1:3))) < 0.00000001 && max(abs(x(4:6))) < 0.00000001
            end_t = 1;
        end

    end %while 

    %variance of unit weight
    sigma0_2 = res'*res/(nof_obs*2 - 6);    

    %store results to EO parameter
    EO = [o p k X0 Y0 Z0];
    
    %display results
    disp('location of projection center (X0, Y0, Z0):')
    disp(EO(4:6))
    disp('rotations (omega, phi,kappa) in gons:')
    disp(rad2gon(EO(1:3)))
    disp('Variance of unit weight:')
    disp(sigma0_2)
    disp('Residuals of image observations:')
    disp(reshape(res,nof_obs(1),2))
end

function [gon]=rad2gon(rad)
    %converts radians into gons
    gon=(rad*400)/(2*pi());
end

function [A,y] = create_A_matrix_and_y_vector(c, o, p, k, X0, Y0, Z0, i_obs, g_obs, nof_obs)
%You need to complete this function

%c=camera constant
%o=omega (current approximation)
%p=phi (current approximation)
%k=kappa (current approximation)
%X0,Y0,Z0 current approximation of the location of projection center
%i_obs=image observations
%g_obs=ground observations
%nof_obs=number of observations

%init design matrix A
A = zeros(nof_obs*2,6);

%init error vector (l-f0)
y = zeros(nof_obs*2,1);

for i = 1:nof_obs %go through all observation
    
    %partial derivatives of collinearity equations
    fx_d_o = make_fx_d_o(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fx_d_p = make_fx_d_p(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fx_d_k = make_fx_d_k(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fx_d_X0 = make_fx_d_X0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fx_d_Y0 = make_fx_d_Y0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fx_d_Z0 = make_fx_d_Z0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_o = make_fy_d_o(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_p = make_fy_d_p(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_k = make_fy_d_k(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_X0 = make_fy_d_X0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_Y0 = make_fy_d_Y0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));
    fy_d_Z0 = make_fy_d_Z0(c,o,p,k,X0,Y0,Z0,g_obs(i,1),g_obs(i,2),g_obs(i,3));      

    %TODO: create here A matrix (two rows with one i value)
  
    
    %values of collinearity equations with current approximate values
    [fx0 fy0] = colli(c,o,p,k,X0,Y0,Z0,g_obs(i,:));

    %TODO: create here y vector (two rows for each i value) i.e.
    %observation-estimated image observations
    
    %notice that in i_obs each row contains one image observation point:
    %x %y, i.e. you'll call it with i_obs(i,1) or i_obs(i,2)
    
    y(i*2-1,1) =
    y(i*2,1) = 
end %for

end


function [x y]=colli(c,o,p,k,X0,Y0,Z0,g_obs)
%computes collinearity equations with current approximate parameters
    r=rot(o,p,k);
    X=g_obs(1,1);
    Y=g_obs(1,2);
    Z=g_obs(1,3);
    U=r(1,1)*(X-X0)+r(1,2)*(Y-Y0)+r(1,3)*(Z-Z0);
    V=r(2,1)*(X-X0)+r(2,2)*(Y-Y0)+r(2,3)*(Z-Z0);
    W=r(3,1)*(X-X0)+r(3,2)*(Y-Y0)+r(3,3)*(Z-Z0);
    x=-c*U/W;
    y=-c*V/W;
    
end


function [R]=rot(o,p,k)
%makes 3D rotation matrix from angles omega, phi, kappa (notice that the indexing of this is
%transpose if compared with e.g. Kraus book)
    R=zeros(3,3);
    R(1,1)= cos(p)*cos(k);
    R(2,1)= -cos(p)*sin(k);
    R(3,1)= sin(p);
    R(1,2)= sin(o)*sin(p)*cos(k)+cos(o)*sin(k);
    R(2,2)= -sin(o)*sin(p)*sin(k)+cos(o)*cos(k);
    R(3,2)= -sin(o)*cos(p);
    R(1,3)= -cos(o)*sin(p)*cos(k)+sin(o)*sin(k);
    R(2,3)= cos(o)*sin(p)*sin(k)+sin(o)*cos(k);
    R(3,3)= cos(o)*cos(p); 
end

%partial derivatives with respect to each unknown parameters
%TODO: copy your results of symbolic partial derivatives here (from file
%partial_derivatives_EO.m after you have created it)

function [fx_d_o]=make_fx_d_o(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fy_d_o]=make_fy_d_o(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fx_d_p]=make_fx_d_p(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fy_d_p]=make_fy_d_p(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fx_d_k]=make_fx_d_k(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fy_d_k]=make_fy_d_k(c,o,p,k,X0,Y0,Z0,X,Y,Z)
 
end

function [fx_d_X0]=make_fx_d_X0(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fy_d_X0]=make_fy_d_X0(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fx_d_Y0]=make_fx_d_Y0(c,o,p,k,X0,Y0,Z0,X,Y,Z)


end

function [fy_d_Y0]=make_fy_d_Y0(c,o,p,k,X0,Y0,Z0,X,Y,Z)
 
end

function [fx_d_Z0]=make_fx_d_Z0(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end

function [fy_d_Z0]=make_fy_d_Z0(c,o,p,k,X0,Y0,Z0,X,Y,Z)

end